@@ -93,23 +93,23 @@ class QQuickContents : public QQuickItemChangeListener
 {
 public:
     QQuickContents(QQuickItem *item);
-    ~QQuickContents();
+    ~QQuickContents() override;
 
     QRectF rectF() const { return m_contents; }
 
-    inline void calcGeometry(QQuickItem *changed = 0);
+    inline void calcGeometry(QQuickItem *changed = nullptr);
     void complete();
 
 protected:
-    void itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change, const QRectF &) Q_DECL_OVERRIDE;
-    void itemDestroyed(QQuickItem *item) Q_DECL_OVERRIDE;
-    void itemChildAdded(QQuickItem *, QQuickItem *) Q_DECL_OVERRIDE;
-    void itemChildRemoved(QQuickItem *, QQuickItem *) Q_DECL_OVERRIDE;
+    void itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change, const QRectF &) override;
+    void itemDestroyed(QQuickItem *item) override;
+    void itemChildAdded(QQuickItem *, QQuickItem *) override;
+    void itemChildRemoved(QQuickItem *, QQuickItem *) override;
     //void itemVisibilityChanged(QQuickItem *item)
 
 private:
-    bool calcHeight(QQuickItem *changed = 0);
-    bool calcWidth(QQuickItem *changed = 0);
+    bool calcHeight(QQuickItem *changed = nullptr);
+    bool calcWidth(QQuickItem *changed = nullptr);
     void updateRect();
 
     QQuickItem *m_item;
@@ -154,7 +154,7 @@ class QQuickItemLayer : public QObject, public QQuickItemChangeListener
 
 public:
     QQuickItemLayer(QQuickItem *item);
-    ~QQuickItemLayer();
+    ~QQuickItemLayer() override;
 
     void classBegin();
     void componentComplete();
@@ -194,11 +194,11 @@ public:
 
     QQuickShaderEffectSource *effectSource() const { return m_effectSource; }
 
-    void itemGeometryChanged(QQuickItem *, QQuickGeometryChange, const QRectF &) Q_DECL_OVERRIDE;
-    void itemOpacityChanged(QQuickItem *) Q_DECL_OVERRIDE;
-    void itemParentChanged(QQuickItem *, QQuickItem *) Q_DECL_OVERRIDE;
-    void itemSiblingOrderChanged(QQuickItem *) Q_DECL_OVERRIDE;
-    void itemVisibilityChanged(QQuickItem *) Q_DECL_OVERRIDE;
+    void itemGeometryChanged(QQuickItem *, QQuickGeometryChange, const QRectF &) override;
+    void itemOpacityChanged(QQuickItem *) override;
+    void itemParentChanged(QQuickItem *, QQuickItem *) override;
+    void itemSiblingOrderChanged(QQuickItem *) override;
+    void itemVisibilityChanged(QQuickItem *) override;
 
     void updateMatrix();
     void updateGeometry();
@@ -255,7 +255,7 @@ public:
     static const QQuickItemPrivate* get(const QQuickItem *item) { return item->d_func(); }
 
     QQuickItemPrivate();
-    ~QQuickItemPrivate();
+    ~QQuickItemPrivate() override;
     void init(QQuickItem *parent);
 
     QQmlListProperty<QObject> data();
@@ -311,7 +311,6 @@ public:
     static void transform_clear(QQmlListProperty<QQuickTransform> *list);
 
     void _q_resourceObjectDeleted(QObject *);
-    void _q_windowChanged(QQuickWindow *w);
     quint64 _q_createJSWrapper(QV4::ExecutionEngine *engine);
 
     enum ChangeType {
@@ -331,7 +330,7 @@ public:
     Q_DECLARE_FLAGS(ChangeTypes, ChangeType)
 
     struct ChangeListener {
-        ChangeListener(QQuickItemChangeListener *l = nullptr, QQuickItemPrivate::ChangeTypes t = 0) : listener(l), types(t), gTypes(QQuickGeometryChange::All) {}
+        ChangeListener(QQuickItemChangeListener *l = nullptr, QQuickItemPrivate::ChangeTypes t = nullptr) : listener(l), types(t), gTypes(QQuickGeometryChange::All) {}
         ChangeListener(QQuickItemChangeListener *l, QQuickGeometryChange gt) : listener(l), types(Geometry), gTypes(gt) {}
         QQuickItemChangeListener *listener;
         QQuickItemPrivate::ChangeTypes types;
@@ -361,13 +360,19 @@ public:
 #endif
         QPointF userTransformOriginPoint;
 
+        // these do not include child items
         int effectRefCount;
         int hideRefCount;
+        // updated recursively for child items as well
+        int recursiveEffectRefCount;
 
         QSGOpacityNode *opacityNode;
         QQuickDefaultClipNode *clipNode;
         QSGRootNode *rootNode;
 
+        // Mask contains() method
+        QMetaMethod maskContains;
+
         QObjectList resourcesList;
 
         // Although acceptedMouseButtons is inside ExtraData, we actually store
@@ -382,6 +387,10 @@ public:
         // 26 bits padding
     };
     QLazilyAllocated<ExtraData> extra;
+    // Contains mask
+    QPointer<QObject> mask;
+    // If the mask is an Item, inform it that it's being used as a mask (true) or is no longer being used (false)
+    virtual void registerAsContainmentMask(QQuickItem * /* maskedItem */, bool /* set */) { }
 
     QQuickAnchors *anchors() const;
     mutable QQuickAnchors *_anchors;
@@ -574,6 +583,8 @@ public:
 
     virtual bool handlePointerEvent(QQuickPointerEvent *, bool avoidExclusiveGrabber = false);
 
+    virtual void setVisible(bool visible);
+
     bool isTransparentForPositioner() const;
     void setTransparentForPositioner(bool trans);
 
@@ -594,9 +605,9 @@ public:
          - (rootNode) (shader effect source's root node)
      */
 
-    QSGOpacityNode *opacityNode() const { return extra.isAllocated()?extra->opacityNode:0; }
-    QQuickDefaultClipNode *clipNode() const { return extra.isAllocated()?extra->clipNode:0; }
-    QSGRootNode *rootNode() const { return extra.isAllocated()?extra->rootNode:0; }
+    QSGOpacityNode *opacityNode() const { return extra.isAllocated()?extra->opacityNode:nullptr; }
+    QQuickDefaultClipNode *clipNode() const { return extra.isAllocated()?extra->clipNode:nullptr; }
+    QSGRootNode *rootNode() const { return extra.isAllocated()?extra->rootNode:nullptr; }
 
     QSGTransformNode *itemNodeInstance;
     QSGNode *paintNode;
@@ -606,6 +617,7 @@ public:
     // A reference from an effect item means that this item is used by the effect, so
     // it should insert a root node.
     void refFromEffectItem(bool hide);
+    void recursiveRefFromEffectItem(int refs);
     void derefFromEffectItem(bool unhide);
 
     void itemChange(QQuickItem::ItemChange, const QQuickItem::ItemChangeData &);
@@ -627,7 +639,7 @@ public:
 class QQuickItemKeyFilter
 {
 public:
-    QQuickItemKeyFilter(QQuickItem * = 0);
+    QQuickItemKeyFilter(QQuickItem * = nullptr);
     virtual ~QQuickItemKeyFilter();
 
     virtual void keyPressed(QKeyEvent *event, bool post);
@@ -649,17 +661,15 @@ class QQuickKeyNavigationAttachedPrivate : public QObjectPrivate
 {
 public:
     QQuickKeyNavigationAttachedPrivate()
-        : QObjectPrivate(),
-          left(0), right(0), up(0), down(0), tab(0), backtab(0),
-          leftSet(false), rightSet(false), upSet(false), downSet(false),
+        : leftSet(false), rightSet(false), upSet(false), downSet(false),
           tabSet(false), backtabSet(false) {}
 
-    QQuickItem *left;
-    QQuickItem *right;
-    QQuickItem *up;
-    QQuickItem *down;
-    QQuickItem *tab;
-    QQuickItem *backtab;
+    QQuickItem *left = nullptr;
+    QQuickItem *right = nullptr;
+    QQuickItem *up = nullptr;
+    QQuickItem *down = nullptr;
+    QQuickItem *tab = nullptr;
+    QQuickItem *backtab = nullptr;
     bool leftSet : 1;
     bool rightSet : 1;
     bool upSet : 1;
@@ -682,7 +692,7 @@ class Q_QUICK_PRIVATE_EXPORT QQuickKeyNavigationAttached : public QObject, publi
     Q_PROPERTY(Priority priority READ priority WRITE setPriority NOTIFY priorityChanged)
 
 public:
-    QQuickKeyNavigationAttached(QObject * = 0);
+    QQuickKeyNavigationAttached(QObject * = nullptr);
 
     QQuickItem *left() const;
     void setLeft(QQuickItem *);
@@ -714,8 +724,8 @@ Q_SIGNALS:
     void priorityChanged();
 
 private:
-    void keyPressed(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
-    void keyReleased(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
+    void keyPressed(QKeyEvent *event, bool post) override;
+    void keyReleased(QKeyEvent *event, bool post) override;
     void setFocusNavigation(QQuickItem *currentItem, const char *dir,
                             Qt::FocusReason reason = Qt::OtherFocusReason);
 };
@@ -727,7 +737,7 @@ class QQuickLayoutMirroringAttached : public QObject
     Q_PROPERTY(bool childrenInherit READ childrenInherit WRITE setChildrenInherit NOTIFY childrenInheritChanged)
 
 public:
-    explicit QQuickLayoutMirroringAttached(QObject *parent = 0);
+    explicit QQuickLayoutMirroringAttached(QObject *parent = nullptr);
 
     bool enabled() const;
     void setEnabled(bool);
@@ -751,7 +761,7 @@ class QQuickEnterKeyAttached : public QObject
     Q_PROPERTY(Qt::EnterKeyType type READ type WRITE setType NOTIFY typeChanged)
 
 public:
-    explicit QQuickEnterKeyAttached(QObject *parent = Q_NULLPTR);
+    explicit QQuickEnterKeyAttached(QObject *parent = nullptr);
 
     Qt::EnterKeyType type() const;
     void setType(Qt::EnterKeyType type);
@@ -770,8 +780,7 @@ class QQuickKeysAttachedPrivate : public QObjectPrivate
 {
 public:
     QQuickKeysAttachedPrivate()
-        : QObjectPrivate(), inPress(false), inRelease(false)
-        , inIM(false), enabled(true), imeItem(0), item(0)
+        : inPress(false), inRelease(false), inIM(false), enabled(true)
     {}
 
     //loop detection
@@ -781,9 +790,9 @@ public:
 
     bool enabled : 1;
 
-    QQuickItem *imeItem;
+    QQuickItem *imeItem = nullptr;
     QList<QQuickItem *> targets;
-    QQuickItem *item;
+    QQuickItem *item = nullptr;
     QQuickKeyEvent theKeyEvent;
 };
 
@@ -797,8 +806,8 @@ class QQuickKeysAttached : public QObject, public QQuickItemKeyFilter
     Q_PROPERTY(Priority priority READ priority WRITE setPriority NOTIFY priorityChanged)
 
 public:
-    QQuickKeysAttached(QObject *parent=0);
-    ~QQuickKeysAttached();
+    QQuickKeysAttached(QObject *parent=nullptr);
+    ~QQuickKeysAttached() override;
 
     bool enabled() const { Q_D(const QQuickKeysAttached); return d->enabled; }
     void setEnabled(bool enabled) {
@@ -819,7 +828,7 @@ public:
         return QQmlListProperty<QQuickItem>(this, d->targets);
     }
 
-    void componentComplete() Q_DECL_OVERRIDE;
+    void componentComplete() override;
 
     static QQuickKeysAttached *qmlAttachedProperties(QObject *);
 
@@ -871,11 +880,11 @@ Q_SIGNALS:
     void volumeDownPressed(QQuickKeyEvent *event);
 
 private:
-    void keyPressed(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
-    void keyReleased(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
+    void keyPressed(QKeyEvent *event, bool post) override;
+    void keyReleased(QKeyEvent *event, bool post) override;
 #if QT_CONFIG(im)
-    void inputMethodEvent(QInputMethodEvent *, bool post) Q_DECL_OVERRIDE;
-    QVariant inputMethodQuery(Qt::InputMethodQuery query) const Q_DECL_OVERRIDE;
+    void inputMethodEvent(QInputMethodEvent *, bool post) override;
+    QVariant inputMethodQuery(Qt::InputMethodQuery query) const override;
 #endif
     void shortcutOverride(QKeyEvent *event) override;
     static QByteArray keyToSignal(int key);
@@ -886,7 +895,7 @@ private:
 Qt::MouseButtons QQuickItemPrivate::acceptedMouseButtons() const
 {
     return ((extra.flag() ? Qt::LeftButton : Qt::MouseButton(0)) |
-            (extra.isAllocated() ? extra->acceptedMouseButtons : Qt::MouseButtons(0)));
+            (extra.isAllocated() ? extra->acceptedMouseButtons : Qt::MouseButtons(nullptr)));
 }
 
 QSGContext *QQuickItemPrivate::sceneGraphContext() const
@@ -908,7 +917,7 @@ void QQuickItemPrivate::markSortedChildrenDirty(QQuickItem *child)
     if (child->z() != 0. || sortedChildItems != &childItems) {
         if (sortedChildItems != &childItems)
             delete sortedChildItems;
-        sortedChildItems = 0;
+        sortedChildItems = nullptr;
     }
 }
 
